import io


class ContextStringIO(io.BytesIO):
    """
    Wrapper to allow use of StringIO via 'with' constructs.
    """

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()
        return False



import avro
import fastavro
import struct
import io
import avro.io
from avro import schema as avroschema
import json
import primed_avro.util


MAGIC_BYTE = 0


class AvroDecoder:
    def __init__(self, schema):
        parsed_schema = avroschema.Parse(json.dumps(schema))
        self._writer = avro.io.DatumWriter(parsed_schema)

    def decode(self, record):
        writer_schema_obj = None  # TODO: where to obtain this?
        reader_schema_obj = None  # TODO: where to obtain this?

        if message is None:
            return None

        if len(message) <= 5:
            raise SerializerError("message is too small to decode")

        avro_reader = avro.io.DatumReader(writer_schema_obj, reader_schema_obj)

        with primed_avro.util.ContextStringIO(message) as payload:
            magic, schema_id = struct.unpack(">bI", payload.read(5))
            if magic != MAGIC_BYTE:
                raise SerializerError("message does not start with magic byte")

            return avro_reader.read(avro.io.BinaryDecoder(payload))


class FastAvroDecoder:
    def __init__(self, schema):
        pass

    def decode(self, schema_id, record):
        pass


class Decoder:
    """
    Allows the user to dynamically specify whether to use the
    FastAvro or regular Avro implementations
    """

    _classmap = {"fastavro": FastAvroDecoder, "avro": AvroDecoder}

    def __init__(self, schema, classname="fastavro"):
        # TODO: decoders not implemented yet
        raise NotImplemented
        return _classmap[classname](schema)



import avro
import fastavro
import struct
import io
import avro.io
from avro import schema as avroschema
import json
import primed_avro.util

MAGIC_BYTE = 0


class AvroEncoder:
    def __init__(self, schema, schemaless=True):
        if not schemaless:
            raise Exception("AvroEncoder does not support encoding schema into records")

        parsed_schema = avroschema.Parse(json.dumps(schema))
        self._encoder = avro.io.DatumWriter(parsed_schema)

    def encode(self, schema_id, record):
        with primed_avro.util.ContextStringIO() as outf:
            # magic byte
            outf.write(struct.pack("b", MAGIC_BYTE))

            # write the schema ID in network byte order (big end)
            outf.write(struct.pack(">I", schema_id))

            # write the record to the rest of it
            # Create an encoder that we'll write to
            encoder = avro.io.BinaryEncoder(outf)

            self._encoder.write(record, encoder)

            return outf.getvalue()


class FastAvroEncoder:
    def __init__(self, schema, schemaless=True):
        self.parsed_schema = fastavro.parse_schema(schema)
        self._encoder = fastavro.schemaless_writer if schemaless else fastavro.writer

    def encode(self, schema_id, record):
        fastavro.validation.validate(record, self.parsed_schema)

        with primed_avro.util.ContextStringIO() as outf:
            outf.write(bytes([MAGIC_BYTE]))  # magic byte
            outf.write((schema_id).to_bytes(4, byteorder="big"))  # schema id
            self._encoder(outf, self.parsed_schema, record)
            return outf.getvalue()


class Encoder:
    """
    Allows the user to dynamically specify whether to use the
    FastAvro or regular Avro implementations
    """

    _classmap = {"fastavro": FastAvroEncoder, "avro": AvroEncoder}

    def __init__(self, schema, schemaless=True, classname="fastavro"):
        return _classmap[classname](schema, schemaless)



import requests
from schema import SchemaMeta


class ConfluentSchemaRegistryClient:
    def __init__(self, url):
        self.url = url

    def get_schema(self, subject, version="latest"):
        r = requests.get(
            "{url}/subjects/{subject}/versions/{version}".format(
                url=self.url, subject=subject, version=version
            ),
            headers={"Content-Type": "application/vnd.schemaregistry.v1+json"},
        )

        if r.status_code == 200:
            js = r.json()

            if "error_code" in js:
                raise Exception(
                    "failed to retrieve schema, error_code: {error_code}, msg: {msg}".format(
                        error_code=js["error_code"], msg=js["msg"]
                    )
                )

            return SchemaMeta(**js)
        else:
            raise Exception(
                "failed to retrieve schema due to connection problems, status code: {status_code}".format(
                    status_code=r.status_code
                )
            )
